@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2009 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2013 Vincent Richard <vincent@vmime.org>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -39,11 +39,10 @@ namespace vmime
 /** Base class for VMime exceptions.
   */
 
-class exception : public std::exception
+class VMIME_EXPORT exception : public std::runtime_error
 {
 private:
 
-	string m_what;
 	exception* m_other;
 
 	exception();
@@ -55,19 +54,14 @@ public:
 
 	virtual ~exception() throw();
 
-	/** Return a description of the error.
+	/** Chain the specified exception with this exception.
 	  *
-	  * @return error message
+	  * @param other next exception in the chain
 	  */
-	const char* what() const throw();
+	void chainException(const exception& other);
 
-	/** Return a description of the error.
-	  *
-	  * @return error message
-	  */
-	const char* what() throw();
-
-	/** Return the next exception in the chain (encapsulated exception).
+	/** Return the next exception in the chain, that is the exception
+	  * that caused this exception. This permits nesting exceptions.
 	  *
 	  * @return next exception in the chain
 	  */
@@ -98,19 +92,19 @@ namespace exceptions
 {
 
 
-class bad_field_type : public vmime::exception
+class VMIME_EXPORT bad_field_value_type : public vmime::exception
 {
 public:
 
-	bad_field_type(const exception& other = NO_EXCEPTION);
-	~bad_field_type() throw();
+	bad_field_value_type(const string& fieldName, const exception& other = NO_EXCEPTION);
+	~bad_field_value_type() throw();
 
 	exception* clone() const;
 	const char* name() const throw();
 };
 
 
-class charset_conv_error : public vmime::exception
+class VMIME_EXPORT charset_conv_error : public vmime::exception
 {
 public:
 
@@ -122,10 +116,22 @@ public:
 };
 
 
+class VMIME_EXPORT illegal_byte_sequence_for_charset : public vmime::exception
+{
+public:
+
+	illegal_byte_sequence_for_charset(const string& what = "", const exception& other = NO_EXCEPTION);
+	~illegal_byte_sequence_for_charset() throw();
+
+	exception* clone() const;
+	const char* name() const throw();
+};
+
+
 /** No encoder has been found for the specified encoding name.
   */
 
-class no_encoder_available : public vmime::exception
+class VMIME_EXPORT no_encoder_available : public vmime::exception
 {
 public:
 
@@ -140,7 +146,7 @@ public:
 /** No algorithm has been found for the specified name.
   */
 
-class no_digest_algorithm_available : public vmime::exception
+class VMIME_EXPORT no_digest_algorithm_available : public vmime::exception
 {
 public:
 
@@ -152,19 +158,7 @@ public:
 };
 
 
-class no_such_parameter : public vmime::exception
-{
-public:
-
-	no_such_parameter(const string& name, const exception& other = NO_EXCEPTION);
-	~no_such_parameter() throw();
-
-	exception* clone() const;
-	const char* name() const throw();
-};
-
-
-class no_such_field : public vmime::exception
+class VMIME_EXPORT no_such_field : public vmime::exception
 {
 public:
 
@@ -176,7 +170,7 @@ public:
 };
 
 
-class no_such_part : public vmime::exception
+class VMIME_EXPORT no_such_part : public vmime::exception
 {
 public:
 
@@ -188,19 +182,7 @@ public:
 };
 
 
-class no_such_mailbox : public vmime::exception
-{
-public:
-
-	no_such_mailbox(const exception& other = NO_EXCEPTION);
-	~no_such_mailbox() throw();
-
-	exception* clone() const;
-	const char* name() const throw();
-};
-
-
-class no_such_message_id : public vmime::exception
+class VMIME_EXPORT no_such_message_id : public vmime::exception
 {
 public:
 
@@ -212,19 +194,7 @@ public:
 };
 
 
-class no_such_address : public vmime::exception
-{
-public:
-
-	no_such_address(const exception& other = NO_EXCEPTION);
-	~no_such_address() throw();
-
-	exception* clone() const;
-	const char* name() const throw();
-};
-
-
-class open_file_error : public vmime::exception
+class VMIME_EXPORT open_file_error : public vmime::exception
 {
 public:
 
@@ -236,7 +206,7 @@ public:
 };
 
 
-class no_factory_available : public vmime::exception
+class VMIME_EXPORT no_factory_available : public vmime::exception
 {
 public:
 
@@ -248,7 +218,7 @@ public:
 };
 
 
-class no_platform_handler : public vmime::exception
+class VMIME_EXPORT no_platform_handler : public vmime::exception
 {
 public:
 
@@ -263,7 +233,7 @@ public:
 /** No expeditor specified.
   */
 
-class no_expeditor : public vmime::exception
+class VMIME_EXPORT no_expeditor : public vmime::exception
 {
 public:
 
@@ -278,7 +248,7 @@ public:
 /** No recipient specified.
   */
 
-class no_recipient : public vmime::exception
+class VMIME_EXPORT no_recipient : public vmime::exception
 {
 public:
 
@@ -290,22 +260,10 @@ public:
 };
 
 
-class no_object_found : public vmime::exception
-{
-public:
-
-	no_object_found(const exception& other = NO_EXCEPTION);
-	~no_object_found() throw();
-
-	exception* clone() const;
-	const char* name() const throw();
-};
-
-
 /** There is no property with that name in the set.
   */
 
-class no_such_property : public vmime::exception
+class VMIME_EXPORT no_such_property : public vmime::exception
 {
 public:
 
@@ -320,7 +278,7 @@ public:
 /** Bad type specified when reading property.
   */
 
-class invalid_property_type : public vmime::exception
+class VMIME_EXPORT invalid_property_type : public vmime::exception
 {
 public:
 
@@ -335,7 +293,7 @@ public:
 /** Bad argument was passed to the function.
   */
 
-class invalid_argument : public vmime::exception
+class VMIME_EXPORT invalid_argument : public vmime::exception
 {
 public:
 
@@ -350,7 +308,7 @@ public:
 /** Underlying operating system error.
   */
 
-class system_error : public vmime::exception
+class VMIME_EXPORT system_error : public vmime::exception
 {
 public:
 
@@ -362,6 +320,21 @@ public:
 };
 
 
+/** The URL is malformed.
+  */
+
+class VMIME_EXPORT malformed_url : public vmime::exception
+{
+public:
+
+	malformed_url(const string& error, const exception& other = NO_EXCEPTION);
+	~malformed_url() throw();
+
+	exception* clone() const;
+	const char* name() const throw();
+};
+
+
 
 #if VMIME_HAVE_MESSAGING_FEATURES
 
@@ -369,7 +342,7 @@ public:
 /** Base class for exceptions thrown by the networking module.
   */
 
-class net_exception : public vmime::exception
+class VMIME_EXPORT net_exception : public vmime::exception
 {
 public:
 
@@ -390,7 +363,7 @@ typedef net_exception messaging_exception;
 /** Socket error.
   */
 
-class socket_exception : public net_exception
+class VMIME_EXPORT socket_exception : public net_exception
 {
 public:
 
@@ -403,11 +376,28 @@ public:
 };
 
 
+/** Socket not connected: you are trying to write to/read from a socket which
+  * is not connected to a peer.
+  */
+
+class VMIME_EXPORT socket_not_connected_exception : public socket_exception
+{
+public:
+
+	socket_not_connected_exception(const string& what = "", const exception& other = NO_EXCEPTION);
+	~socket_not_connected_exception() throw();
+
+	exception* clone() const;
+	const char* name() const throw();
+
+};
+
+
 /** Error while connecting to the server: this may be a DNS resolution error
   * or a connection error (for example, time-out while connecting).
   */
 
-class connection_error : public socket_exception
+class VMIME_EXPORT connection_error : public socket_exception
 {
 public:
 
@@ -422,7 +412,7 @@ public:
 /** Server did not initiated the connection correctly.
   */
 
-class connection_greeting_error : public net_exception
+class VMIME_EXPORT connection_greeting_error : public net_exception
 {
 public:
 
@@ -444,7 +434,7 @@ private:
   * or password, or wrong authentication method).
   */
 
-class authentication_error : public net_exception
+class VMIME_EXPORT authentication_error : public net_exception
 {
 public:
 
@@ -465,7 +455,7 @@ private:
 /** Option not supported.
   */
 
-class unsupported_option : public net_exception
+class VMIME_EXPORT unsupported_option : public net_exception
 {
 public:
 
@@ -477,26 +467,11 @@ public:
 };
 
 
-/** No service available for this protocol.
-  */
-
-class no_service_available : public net_exception
-{
-public:
-
-	no_service_available(const string& proto = "", const exception& other = NO_EXCEPTION);
-	~no_service_available() throw();
-
-	exception* clone() const;
-	const char* name() const throw();
-};
-
-
 /** The current state of the object does not permit to execute the
   * operation (for example, you try to close a folder which is not open).
   */
 
-class illegal_state : public net_exception
+class VMIME_EXPORT illegal_state : public net_exception
 {
 public:
 
@@ -511,7 +486,7 @@ public:
 /** Folder not found (does not exist).
   */
 
-class folder_not_found : public net_exception
+class VMIME_EXPORT folder_not_found : public net_exception
 {
 public:
 
@@ -523,10 +498,25 @@ public:
 };
 
 
+/** Folder is already open in the same session.
+  */
+
+class VMIME_EXPORT folder_already_open : public net_exception
+{
+public:
+
+	folder_already_open(const exception& other = NO_EXCEPTION);
+	~folder_already_open() throw();
+
+	exception* clone() const;
+	const char* name() const throw();
+};
+
+
 /** Message not found (does not exist).
   */
 
-class message_not_found : public net_exception
+class VMIME_EXPORT message_not_found : public net_exception
 {
 public:
 
@@ -541,7 +531,7 @@ public:
 /** Operation not supported by the underlying protocol.
   */
 
-class operation_not_supported : public net_exception
+class VMIME_EXPORT operation_not_supported : public net_exception
 {
 public:
 
@@ -556,7 +546,7 @@ public:
 /** The operation timed out (time-out delay is elapsed).
   */
 
-class operation_timed_out : public net_exception
+class VMIME_EXPORT operation_timed_out : public net_exception
 {
 public:
 
@@ -571,7 +561,7 @@ public:
 /** The operation has been cancelled.
   */
 
-class operation_cancelled : public net_exception
+class VMIME_EXPORT operation_cancelled : public net_exception
 {
 public:
 
@@ -587,7 +577,7 @@ public:
   * the requested object.
   */
 
-class unfetched_object : public net_exception
+class VMIME_EXPORT unfetched_object : public net_exception
 {
 public:
 
@@ -602,7 +592,7 @@ public:
 /** The service is not currently connected.
   */
 
-class not_connected : public net_exception
+class VMIME_EXPORT not_connected : public net_exception
 {
 public:
 
@@ -617,7 +607,7 @@ public:
 /** The service is already connected (must disconnect before).
   */
 
-class already_connected : public net_exception
+class VMIME_EXPORT already_connected : public net_exception
 {
 public:
 
@@ -632,7 +622,7 @@ public:
 /** Illegal operation: cannot run this operation on the object.
   */
 
-class illegal_operation : public net_exception
+class VMIME_EXPORT illegal_operation : public net_exception
 {
 public:
 
@@ -647,7 +637,7 @@ public:
 /** Command error: operation failed (this is specific to the underlying protocol).
   */
 
-class command_error : public net_exception
+class VMIME_EXPORT command_error : public net_exception
 {
 public:
 
@@ -681,7 +671,7 @@ private:
 /** The server returned an invalid response.
   */
 
-class invalid_response : public net_exception
+class VMIME_EXPORT invalid_response : public net_exception
 {
 public:
 
@@ -715,7 +705,7 @@ private:
 /** Partial fetch is not supported by the underlying protocol.
   */
 
-class partial_fetch_not_supported : public net_exception
+class VMIME_EXPORT partial_fetch_not_supported : public net_exception
 {
 public:
 
@@ -727,25 +717,10 @@ public:
 };
 
 
-/** The URL is malformed.
-  */
-
-class malformed_url : public net_exception
-{
-public:
-
-	malformed_url(const string& error, const exception& other = NO_EXCEPTION);
-	~malformed_url() throw();
-
-	exception* clone() const;
-	const char* name() const throw();
-};
-
-
 /** Folder name is invalid.
   */
 
-class invalid_folder_name : public net_exception
+class VMIME_EXPORT invalid_folder_name : public net_exception
 {
 public:
 
@@ -766,7 +741,7 @@ public:
 /** Base class for exceptions thrown by the filesystem features.
   */
 
-class filesystem_exception : public vmime::exception
+class VMIME_EXPORT filesystem_exception : public vmime::exception
 {
 public:
 
@@ -791,7 +766,7 @@ private:
 /** File is not a directory.
   */
 
-class not_a_directory : public filesystem_exception
+class VMIME_EXPORT not_a_directory : public filesystem_exception
 {
 public:
 
@@ -806,7 +781,7 @@ public:
 /** File not found.
   */
 
-class file_not_found : public filesystem_exception
+class VMIME_EXPORT file_not_found : public filesystem_exception
 {
 public:
 
@@ -824,7 +799,7 @@ public:
 /** Authentication exception.
   */
 
-class authentication_exception : public vmime::exception
+class VMIME_EXPORT authentication_exception : public vmime::exception
 {
 public:
 
@@ -839,7 +814,7 @@ public:
 /** The requested information cannot be provided.
   */
 
-class no_auth_information : public authentication_exception
+class VMIME_EXPORT no_auth_information : public authentication_exception
 {
 public:
 
@@ -857,7 +832,7 @@ public:
 /** Base class for exceptions thrown by SASL module.
   */
 
-class sasl_exception : public authentication_exception
+class VMIME_EXPORT sasl_exception : public authentication_exception
 {
 public:
 
@@ -872,7 +847,7 @@ public:
 /** No mechanism is registered with the specified name.
   */
 
-class no_such_mechanism : public sasl_exception
+class VMIME_EXPORT no_such_mechanism : public sasl_exception
 {
 public:
 
@@ -893,7 +868,7 @@ public:
 /** Base class for exceptions thrown by TLS module.
   */
 
-class tls_exception : public vmime::exception
+class VMIME_EXPORT tls_exception : public vmime::exception
 {
 public:
 
@@ -905,42 +880,6 @@ public:
 };
 
 
-class certificate_exception : public tls_exception
-{
-public:
-
-	certificate_exception(const string& what, const exception& other = NO_EXCEPTION);
-	~certificate_exception() throw();
-
-	exception* clone() const;
-	const char* name() const throw();
-};
-
-
-class certificate_verification_exception : public certificate_exception
-{
-public:
-
-	certificate_verification_exception(const string& what, const exception& other = NO_EXCEPTION);
-	~certificate_verification_exception() throw ();
-
-	exception* clone() const;
-	const char* name() const throw ();
-};
-
-
-class unsupported_certificate_type : public certificate_exception
-{
-public:
-
-	unsupported_certificate_type(const string& type, const exception& other = NO_EXCEPTION);
-	~unsupported_certificate_type() throw ();
-
-	exception* clone() const;
-	const char* name() const throw ();
-};
-
-
 #endif // VMIME_HAVE_TLS_SUPPORT
 
 
