@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2009 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2013 Vincent Richard <vincent@vmime.org>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -25,6 +25,12 @@
 #define VMIME_NET_DEFAULTCONNECTIONINFOS_HPP_INCLUDED
 
 
+#include "vmime/config.hpp"
+
+
+#if VMIME_HAVE_MESSAGING_FEATURES
+
+
 #include "vmime/net/connectionInfos.hpp"
 
 
@@ -34,7 +40,7 @@ namespace net {
 
 /** Information about the connection used by a service.
   */
-class defaultConnectionInfos : public connectionInfos
+class VMIME_EXPORT defaultConnectionInfos : public connectionInfos
 {
 public:
 
@@ -54,5 +60,7 @@ private:
 } // vmime
 
 
+#endif // VMIME_HAVE_MESSAGING_FEATURES
+
 #endif // VMIME_NET_DEFAULTCONNECTIONINFOS_HPP_INCLUDED
 
