@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2009 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2013 Vincent Richard <vincent@vmime.org>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -25,15 +25,17 @@
 #define VMIME_PLATFORMS_POSIX_POSIXCHILDPROCESS_HPP_INCLUDED
 
 
+#include "vmime/config.hpp"
+
+
+#if VMIME_PLATFORM_IS_POSIX && VMIME_HAVE_FILESYSTEM_FEATURES
+
+
 #include "vmime/utility/childProcess.hpp"
 
 #include <sys/types.h>
 #include <signal.h>
 
-#ifdef __OpenBSD__
-#include <sys/types.h>
-#include <signal.h>
-#endif
 
 namespace vmime {
 namespace platforms {
@@ -49,8 +51,8 @@ public:
 
 	void start(const std::vector <string> args, const int flags = 0);
 
-	ref <utility::outputStream> getStdIn();
-	ref <utility::inputStream> getStdOut();
+	shared_ptr <utility::outputStream> getStdIn();
+	shared_ptr <utility::inputStream> getStdOut();
 
 	void waitForFinish();
 
@@ -59,8 +61,8 @@ private:
 	utility::file::path m_processPath;
 	bool m_started;
 
-	ref <utility::outputStream> m_stdIn;
-	ref <utility::inputStream> m_stdOut;
+	shared_ptr <utility::outputStream> m_stdIn;
+	shared_ptr <utility::inputStream> m_stdOut;
 
 	sigset_t m_oldProcMask;
 	pid_t m_pid;
@@ -75,7 +77,7 @@ class posixChildProcessFactory : public utility::childProcessFactory
 {
 public:
 
-	ref <utility::childProcess> create(const utility::file::path& path) const;
+	shared_ptr <utility::childProcess> create(const utility::file::path& path) const;
 };
 
 
@@ -84,5 +86,7 @@ public:
 } // vmime
 
 
+#endif // VMIME_PLATFORM_IS_POSIX && VMIME_HAVE_FILESYSTEM_FEATURES
+
 #endif // VMIME_PLATFORMS_POSIX_POSIXCHILDPROCESS_HPP_INCLUDED
 
