@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2009 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2013 Vincent Richard <vincent@vmime.org>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -25,6 +25,12 @@
 #define VMIME_SECURITY_SASL_BUILTINSASLMECHANISM_HPP_INCLUDED
 
 
+#include "vmime/config.hpp"
+
+
+#if VMIME_HAVE_MESSAGING_FEATURES && VMIME_HAVE_SASL_SUPPORT
+
+
 #include "vmime/security/sasl/SASLMechanism.hpp"
 
 
@@ -39,34 +45,36 @@ class SASLContext;
 /** A built-in authentication mechanism that relies on
   * the GNU SASL library.
   */
-class builtinSASLMechanism : public SASLMechanism
+class VMIME_EXPORT builtinSASLMechanism : public SASLMechanism
 {
 public:
 
-	builtinSASLMechanism(ref <SASLContext> ctx, const string& name);
+	builtinSASLMechanism(shared_ptr <SASLContext> ctx, const string& name);
 	~builtinSASLMechanism();
 
 
 	const string getName() const;
 
-	bool step(ref <SASLSession> sess,
-		 const byte_t* challenge, const int challengeLen,
-		 byte_t** response, int* responseLen);
+	bool step(shared_ptr <SASLSession> sess,
+		 const byte_t* challenge, const size_t challengeLen,
+		 byte_t** response, size_t* responseLen);
 
 	bool isComplete() const;
 
-	void encode(ref <SASLSession> sess,
-		const byte_t* input, const int inputLen,
-		byte_t** output, int* outputLen);
+	bool hasInitialResponse() const;
 
-	void decode(ref <SASLSession> sess,
-		const byte_t* input, const int inputLen,
-		byte_t** output, int* outputLen);
+	void encode(shared_ptr <SASLSession> sess,
+		const byte_t* input, const size_t inputLen,
+		byte_t** output, size_t* outputLen);
+
+	void decode(shared_ptr <SASLSession> sess,
+		const byte_t* input, const size_t inputLen,
+		byte_t** output, size_t* outputLen);
 
 private:
 
 	/** SASL context */
-	ref <SASLContext> m_context;
+	shared_ptr <SASLContext> m_context;
 
 	/** Mechanism name */
 	const string m_name;
@@ -81,5 +89,7 @@ private:
 } // vmime
 
 
+#endif // VMIME_HAVE_MESSAGING_FEATURES && VMIME_HAVE_SASL_SUPPORT
+
 #endif // VMIME_SECURITY_SASL_BUILTINSASLMECHANISM_HPP_INCLUDED
 
