@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2009 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2013 Vincent Richard <vincent@vmime.org>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -38,7 +38,7 @@ namespace utility {
 /** Abstract representation of a path (filesystem, mailbox, etc).
   */
 
-class path : public object
+class VMIME_EXPORT path : public object
 {
 public:
 
@@ -80,14 +80,14 @@ public:
 	  * @param pos position
 	  * @return component at position 'pos'
 	  */
-	const component& getComponentAt(const int pos) const;
+	const component& getComponentAt(const size_t pos) const;
 
 	/** Return the component at the specified position.
 	  *
 	  * @param pos position
 	  * @return component at position 'pos'
 	  */
-	component& getComponentAt(const int pos);
+	component& getComponentAt(const size_t pos);
 
 	/** Test whether this path is empty (root).
 	  *
@@ -117,21 +117,21 @@ public:
 	  *
 	  * @return number of components
 	  */
-	int getSize() const;
+	size_t getSize() const;
 
 	/** Return the specified component of the path (const version).
 	  *
 	  * @param x index of the component
 	  * @return component at the specified index
 	  */
-	const component& operator[](const int x) const;
+	const component& operator[](const size_t x) const;
 
 	/** Return the specified component of the path (non-const version).
 	  *
 	  * @param x index of the component
 	  * @return component at the specified index
 	  */
-	component& operator[](const int x);
+	component& operator[](const size_t x);
 
 	/** Test whether this path is a direct parent of another one.
 	  *
@@ -158,6 +158,25 @@ public:
 	  */
 	void renameParent(const path& oldPath, const path& newPath);
 
+	/** Construct a new path from a string.
+	  *
+	  * @param str string representation of the path
+	  * @param sep separator string (eg: "/")
+	  * @param cset charset in which the path is encoded (use the value returned by
+	  * vmime::charset::getLocalCharset() to use the default charset of your system)
+	  * @return a new path corresponding to the specified string
+	  */
+	static path fromString(const string& str, const string& sep, const charset& cset);
+
+	/** Returns a string representation of this path.
+	  *
+	  * @param sep separator string (eg: "/")
+	  * @param cset charset in which to encode the components (use the value returned by
+	  * vmime::charset::getLocalCharset() to use the default charset of your system)
+	  * @return a string representing this path
+	  */
+	const string toString(const string& sep, const charset& cset) const;
+
 private:
 
 	list m_list;
