@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2009 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2013 Vincent Richard <vincent@vmime.org>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -38,10 +38,54 @@ namespace utility {
 /** Miscellaneous functions related to strings.
   */
 
-class stringUtils
+class VMIME_EXPORT stringUtils
 {
 public:
 
+	/** Makes a string from bytes.
+	  *
+	  * @param data pointer to buffer containing data
+	  * @param count number of bytes to use from buffer
+	  * @return a string object containing a copy of the specified data
+	  */
+	static const string makeStringFromBytes(const byte_t* data, const size_t count)
+	{
+		return string(reinterpret_cast <const char*>(data), count);
+	}
+
+	/** Casts a string to bytes.
+	  *
+	  * @param str string
+	  * @return pointer to the first byte of the string
+	  */
+	static const byte_t* bytesFromString(const string& str)
+	{
+		return reinterpret_cast <const byte_t*>(str.data());
+	}
+
+	/** Casts a NULL-terminated string to bytes.
+	  *
+	  * @param str string
+	  * @return pointer to the first byte of the string
+	  */
+	static const byte_t* bytesFromString(const char* str)
+	{
+		return reinterpret_cast <const byte_t*>(str);
+	}
+
+	/** Appends bytes to a string.
+	  *
+	  * @param str string to which append data
+	  * @param data pointer to buffer containing data
+	  * @param count number of bytes to use from buffer
+	  * @return a reference to modified string
+	  */
+	static string& appendBytesToString(string& str, const byte_t* data, const size_t count)
+	{
+		str.append(reinterpret_cast <const char*>(data), count);
+		return str;
+	}
+
 	/** Test two strings for equality (case insensitive).
 	  * \warning Use this with ASCII-only strings.
 	  *
@@ -50,7 +94,7 @@ public:
 	  * @param n length of the second string
 	  * @return true if the two strings compare equally, false otherwise
 	  */
-	static bool isStringEqualNoCase(const string& s1, const char* s2, const string::size_type n);
+	static bool isStringEqualNoCase(const string& s1, const char* s2, const size_t n);
 
 	/** Test two strings for equality (case insensitive).
 	  * \warning Use this with ASCII-only strings.
@@ -70,7 +114,7 @@ public:
 	  * @param n length of the second string
 	  * @return true if the two strings compare equally, false otherwise
 	  */
-	static bool isStringEqualNoCase(const string::const_iterator begin, const string::const_iterator end, const char* s, const string::size_type n);
+	static bool isStringEqualNoCase(const string::const_iterator begin, const string::const_iterator end, const char* s, const size_t n);
 
 	/** Transform all the characters in a string to lower-case.
 	  * \warning Use this with ASCII-only strings.
@@ -102,7 +146,15 @@ public:
 	  * @param end end position
 	  * @return number of ASCII characters
 	  */
-	static string::size_type countASCIIchars(const string::const_iterator begin, const string::const_iterator end);
+	static size_t countASCIIchars(const string::const_iterator begin, const string::const_iterator end);
+
+	/** Returns whether the specified string is composed exclusively
+	  * of 7-bit ASCII characters.
+	  *
+	  * @param str string to test
+	  * @return true if the string is ASCII-only, false otherwise
+	  */
+	static bool is7bit(const string& str);
 
 	/** Returns the position of the first non 7-bit US-ASCII character in a string.
 	  *
@@ -110,7 +162,7 @@ public:
 	  * @param end end position
 	  * @return position since begin, or string::npos
 	  */
-	static string::size_type findFirstNonASCIIchar(const string::const_iterator begin, const string::const_iterator end);
+	static size_t findFirstNonASCIIchar(const string::const_iterator begin, const string::const_iterator end);
 
 	/** Convert the specified value to a string value.
 	  *
@@ -149,10 +201,50 @@ public:
 
 	/** Unquote the specified string and transform escaped characters.
 	  *
-	  * @param string from which to remove quotes
+	  * @param str string from which to remove quotes
 	  * @return unquoted string
 	  */
 	static const string unquote(const string& str);
+
+	/** Determines whether the specified string needs to be quoted.
+	  *
+	  * @param str string to test
+	  * @param specialChars list of characters that will cause the
+	  * string to be quoted
+	  * @return true if the string needs to be quoted, false otherwise
+	  */
+	static bool needQuoting(const string& str,
+		const string& specialChars = " \t\"(),:;<>@[\\]");
+
+	/** Quotes the specified string.
+	  *
+	  * @param str string to quote
+	  * @param escapeSpecialChars list of characters that will be escaped
+	  * @param escapeChar character that will be used for escaping (eg. '\')
+	  * @return quoted string
+	  */
+	static string quote(const string& str, const string& escapeSpecialChars, const string& escapeChar);
+
+	/** Return whether the specified string is a valid host name
+	  * or domain name.
+	  *
+	  * @param hostname string to test
+	  * @return true if the string is a valid host name or domain
+	  * name, or false otherwise
+	  */
+	static bool isValidHostname(const vmime::string& hostname);
+
+	/** Return whether the specified string is a valid fully
+	  * qualified domain name (FQDN).
+	  *
+	  * @param fqdn string to test
+	  * @return true if the string seems to be a FQDN, false otherwise
+	  */
+	static bool isValidFQDN(const vmime::string& fqdn);
+
+private:
+
+	static bool isValidFQDNImpl(const vmime::string& fqdn, short* minNumberOfDots);
 };
 
 
